package Profits;

use Mojo::Base 'Mojolicious';
use Mojo::Pg;
use Mojo::Redis;
use File::Find;
use Path::Tiny qw( path );
use Profits::Schema;

use constant SECRETS => '/run/secrets';

sub startup {
    my $self = shift;

    # env z docker secrets (maji prednost!)
    find(sub { $ENV{$_} = path(SECRETS . "/$_")->slurp_utf8 }, SECRETS);

    # Nacteni konfigurace
    my $cfg = {};
    $self->helper( cfg => sub { return $cfg; } );

    KEY:
    foreach my $key ( keys %ENV ) {
        if ( $key =~ /^CFG_(.+)/i ) {
            $cfg->{lc($1)} = $ENV{$key};
        }
    }

    $self->plugin('SentrySDK',  {
        dsn     => $cfg->{sentry_dsn},
        traces_sample_rate => 1,
    }) if $cfg->{sentry_dsn};

    $self->plugin('Profits::Helpers::Core');
    $self->plugin('Profits::Helpers::Auth');

    my $redis = Mojo::Redis->new( 'redis://' . $cfg->{redis} );
    $self->helper( redis => sub { return $redis; } );

    # migrace schematu
    my $pg = Mojo::Pg->new
        ->dsn($cfg->{db_dsn})
        ->username($cfg->{db_username})
        ->password($cfg->{db_password})
    ;
    if ($cfg->{test}) {
        $pg->search_path(['test']);
        $pg->db->query('drop schema if exists test cascade');
        $pg->db->query('create schema test');
    }
    $pg->migrations->from_file($self->home . '/sql/migrations.sql');
    $pg->migrations->migrate();
    $self->helper( pg => sub { return $pg; } );

    # Spojeni s databazi
    my $schema = Profits::Schema->connect({
        dsn            => $cfg->{db_dsn},
        user           => $cfg->{db_username},
        password       => $cfg->{db_password},
        AutoCommit     => 1,
        quote_char     => '"',
        name_sep       => '.',
        pg_enable_utf8 => 1,
        on_connect_do  => [
            "set timezone to 'Europe/Prague'",
        ],
    });
    if ( $cfg->{test} ) {
        $schema->storage->dbh->do("set search_path to test") ;
    }
    $self->helper( schema => sub { return $schema; } );

    $self->plugin("OpenAPI" => {
        url      => $self->home->rel_file("openapi.yaml"),
        schema => 'v3',
        plugins                        => [qw(+Cors +Security)],
        default_response_codes         => [400, 401, 403, 404, 500, 501],

        security => {
            Bearer => sub {
                my ($c, $definition, $scopes, $cb ) = @_;

                my $key = $c->req->headers->authorization;

                return $c->$cb('Authorization header not present') if ! $key;
                return $c->$cb('Unsupported authorization type') if $key !~ s/Bearer\s+//i;

                $c->oauth_token($key);

                if (! $c->user ) {
                    return $c->$cb('Invalid user');
                }

                my $user = $c->schema->resultset('User')->find_or_create(
                    $c->user, { key => 'uuid'}
                );
                $c->stash->{user} = $user;
            }
        }
    });

    # defautni globalni promenne ve stash
    $self->defaults(
        openapi_cors_allowed_origins => ['*'],
    );

}

1;
